@@ -15,7 +15,6 @@
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 import pecan
-import six
 
 from zun.api.controllers import base
 from zun.api.controllers import link
@@ -41,15 +40,6 @@ CONF = zun.conf.CONF
 LOG = logging.getLogger(__name__)
 
 
-def _get_capsule(capsule_ident):
-    """Get capsule by name or UUID"""
-    capsule = api_utils.get_resource('Capsule', capsule_ident)
-    if not capsule:
-        pecan.abort(404, ('Not found; the capsule you requested '
-                          'does not exist.'))
-    return capsule
-
-
 def check_policy_on_capsule(capsule, action):
     context = pecan.request.context
     policy.enforce(context, action, capsule, action=action)
@@ -58,7 +48,7 @@ def check_policy_on_capsule(capsule, action):
 def check_capsule_template(tpl):
     # TODO(kevinz): add volume spec check
     tpl_json = tpl
-    if isinstance(tpl, six.string_types):
+    if isinstance(tpl, str):
         try:
             tpl_json = jsonutils.loads(tpl)
         except Exception as e:
@@ -132,7 +122,7 @@ class CapsuleController(base.Controller):
     @base.Controller.api_version("1.32")  # noqa
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
-    def get_all(self, **kwargs):
+    def get_all(self, **kwargs):  # noqa
         '''Retrieve a list of capsules.'''
         return self._do_get_all(**kwargs)
 
@@ -182,7 +172,7 @@ class CapsuleController(base.Controller):
     @api_utils.enforce_content_types(['application/json'])
     @exception.wrap_pecan_controller_exception
     @validation.validated(schema.capsule_create)
-    def post(self, **capsule_dict):
+    def post(self, **capsule_dict):  # noqa
         """Create a new capsule.
 
         :param capsule_dict: a capsule within the request body.
@@ -235,6 +225,7 @@ class CapsuleController(base.Controller):
         if metadata_info:
             new_capsule.name = metadata_info.get('name', None)
             new_capsule.labels = metadata_info.get('labels', None)
+            new_capsule.annotations = metadata_info.get('annotations', None)
 
         # create the capsule in DB so that it generates a 'id'
         new_capsule.save()
@@ -271,8 +262,12 @@ class CapsuleController(base.Controller):
         merged_containers_spec = init_containers_spec + containers_spec
         for container_spec in merged_containers_spec:
             if container_spec.get('image_pull_policy'):
-                policy.enforce(context, "container:create:image_pull_policy",
-                               action="container:create:image_pull_policy")
+                if not policy.enforce(
+                        context, "container:create:image_pull_policy",
+                        action="container:create:image_pull_policy",
+                        do_raise=False):
+                    LOG.info("Policy doesn't support image_pull_policy")
+                    container_spec.pop('image_pull_policy')
             container_dict = container_spec
             container_dict['project_id'] = context.project_id
             container_dict['user_id'] = context.user_id
@@ -345,7 +340,7 @@ class CapsuleController(base.Controller):
         kwargs['extra_spec'] = extra_spec
         kwargs['requested_networks'] = requested_networks
         kwargs['requested_volumes'] = requested_volumes
-        kwargs['run'] = True
+        kwargs['run'] = False
         compute_api.container_create(context, new_capsule, **kwargs)
         # Set the HTTP Location Header
         pecan.response.location = link.build_url('capsules',
@@ -369,7 +364,7 @@ class CapsuleController(base.Controller):
     @base.Controller.api_version("1.32")  # noqa
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
-    def get_one(self, capsule_ident):
+    def get_one(self, capsule_ident):  # noqa
         """Retrieve information about the given capsule.
 
         :param capsule_ident: UUID or name of a capsule.
@@ -378,7 +373,7 @@ class CapsuleController(base.Controller):
 
     def _do_get_one(self, capsule_ident, legacy_api_version=False):
         context = pecan.request.context
-        capsule = _get_capsule(capsule_ident)
+        capsule = api_utils.get_resource('Capsule', capsule_ident)
         check_policy_on_capsule(capsule.as_dict(), "capsule:get")
         return view.format_capsule(pecan.request.host_url, capsule, context,
                                    legacy_api_version=legacy_api_version)
@@ -395,13 +390,18 @@ class CapsuleController(base.Controller):
             policy.enforce(context, "capsule:delete_all_projects",
                            action="capsule:delete_all_projects")
             context.all_projects = True
-        capsule = _get_capsule(capsule_ident)
+        capsule = api_utils.get_resource('Capsule', capsule_ident)
         check_policy_on_capsule(capsule.as_dict(), "capsule:delete")
         compute_api = pecan.request.compute_api
         capsule.task_state = consts.CONTAINER_DELETING
         capsule.save(context)
-        compute_api.container_stop(context, capsule, 10)
-        compute_api.container_delete(context, capsule)
+        if capsule.host:
+            compute_api.container_delete(context, capsule)
+        else:
+            merged_containers = capsule.containers + capsule.init_containers
+            for container in merged_containers:
+                container.destroy(context)
+            capsule.destroy(context)
         pecan.response.status = 204
 
     def _generate_name_for_capsule_container(self, new_capsule):
@@ -477,7 +477,7 @@ class CapsuleController(base.Controller):
                     cinder_api.delete_volume(volume.id)
                 except Exception as exc:
                     LOG.error('Error on deleting volume "%s": %s.',
-                              volume.id, six.text_type(exc))
+                              volume.id, str(exc))
 
             # Since the container and capsule database model has been created,
             # we need to delete them here due to the volume create failed.
