@@ -11,7 +11,6 @@
 # under the License.
 
 import contextlib
-import six
 import sys
 import tarfile
 
@@ -23,6 +22,7 @@ from oslo_utils import encodeutils
 from zun.common import consts
 from zun.common import exception
 from zun.common.i18n import _
+from zun.common import utils
 import zun.conf
 
 
@@ -45,8 +45,8 @@ def docker_client():
             **client_kwargs
         )
     except errors.APIError as e:
-        desired_exc = exception.DockerError(error_msg=six.text_type(e))
-        six.reraise(type(desired_exc), desired_exc, sys.exc_info()[2])
+        desired_exc = exception.DockerError(error_msg=str(e))
+        utils.reraise(type(desired_exc), desired_exc, sys.exc_info()[2])
 
 
 class DockerHTTPClient(docker.APIClient):
@@ -99,5 +99,5 @@ class DockerHTTPClient(docker.APIClient):
         except errors.APIError as e:
             if "process not found for container" in str(e):
                 raise exception.Invalid(_(
-                    "no such exec instance: %s") % six.text_type(e))
+                    "no such exec instance: %s") % str(e))
             raise
